package com.xy.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.dto.DeviceFaultInfoDto;
import com.xy.entity.DeviceFaultInfo;
import com.xy.entity.DeviceFaultLog;
import com.xy.mapper.DeviceFaultInfoMapper;
import com.xy.utils.Emptys;
import com.xy.utils.MybatisPlusQuery;
import com.xy.utils.PageBean;
import com.xy.utils.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;
import java.util.List;

import static com.xy.utils.Beans.copy;
import static com.xy.utils.PlusBeans.toIPage;
import static com.xy.utils.PlusBeans.toPageBean;


/**
 * <p>
 * 设备故障信息 服务实现类
 * </p>
 *
 * @author 谭斌
 * @since 2024-01-08
 */
@Service
@RequiredArgsConstructor
@Api(tags = "设备故障信息")
public class DeviceFaultInfoServiceImpl extends ServiceImpl<DeviceFaultInfoMapper, DeviceFaultInfo> implements DeviceFaultInfoService {

    private final DeviceFaultLogServiceImpl deviceFaultLogService;

    @PostMapping("obj")
    @ApiOperation("对象查询")
    public R<DeviceFaultInfoDto.Vo> obj(@RequestBody DeviceFaultInfoDto.Vo vo) {
        DeviceFaultInfoDto.SelectList selectList = copy(DeviceFaultInfoDto.SelectList.class, vo);
        List<DeviceFaultInfoDto.Vo> list = list(selectList).getData();
        if (Emptys.check(list)) {
            return R.ok(list.get(0));
        }
        return R.ok();
    }

    @PostMapping("list")
    @ApiOperation("集合查询")
    public R<List<DeviceFaultInfoDto.Vo>> list(@RequestBody DeviceFaultInfoDto.SelectList selectList) {
        LambdaQueryWrapper<DeviceFaultInfo> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(selectList, DeviceFaultInfo.class).build();
        List<DeviceFaultInfo> list = list(lambdaQueryWrapper);
        return R.ok(copy(DeviceFaultInfoDto.Vo.class, list));
    }

    @PostMapping("page")
    @ApiOperation("分页查询")
    public R<PageBean<DeviceFaultInfoDto.Vo>> page(@RequestBody DeviceFaultInfoDto.Page page) {
        PageBean pageBean = page.getPage();
        LambdaQueryWrapper<DeviceFaultInfo> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(page, DeviceFaultInfo.class).build();
        IPage<DeviceFaultInfo> iPage = page(toIPage(pageBean), lambdaQueryWrapper);
        return R.ok(toPageBean(DeviceFaultInfoDto.Vo.class, iPage));
    }

    @PostMapping("save")
    @ApiOperation("添加")
    public R save(@RequestBody @Validated DeviceFaultInfoDto.Save save) {
        DeviceFaultInfo saveInfo = copy(DeviceFaultInfo.class, save);
        save(saveInfo);
        return R.ok();
    }

    @PostMapping("update")
    @ApiOperation("修改")
    public R update(@RequestBody @Validated DeviceFaultInfoDto.Update update) {
        DeviceFaultInfo updateInfo = copy(DeviceFaultInfo.class, update);
        updateById(updateInfo);
        return R.ok();
    }

    @PostMapping("resolve")
    @ApiOperation("标记已解决")
    public R resolve(@RequestBody @Validated DeviceFaultInfoDto.Update update) {
        DeviceFaultInfo updateInfo = copy(DeviceFaultInfo.class, update);
        updateInfo.setState(true);
        boolean b = updateById(updateInfo);
        if (b) {
            //清理日志 故障信息表标记已解决，清除日志表中对应设备事件在标记已解决时间之前所有事件。
            deviceFaultLogService.remove(Wrappers.<DeviceFaultLog>lambdaUpdate().le(DeviceFaultLog::getEventTime, LocalDateTime.now()));
        }
        return R.ok();
    }
}
